package	jp.co.sds_corp.hibiya.monitor;

import java.awt.Component;
import java.awt.Dialog;
import java.awt.Frame;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.ResourceBundle;
import java.util.Vector;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableColumn;

import jp.co.sds_corp.hibiya.action.IAction;
import jp.co.sds_corp.hibiya.filter.EventFilter;
import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;
import jp.co.sds_corp.util.window.WindowUtility;

/**
* {@link IAction}の設定パネルです。
*/
public class ActionPanel extends JPanel
{
	private static final long serialVersionUID = 1L;
	CmnLogger logger = CmnLogFactory.getInstance().getLogger(this);
	Window		owner;
	SetupParam	setup;
	Vector<IAction>		vecAction = new Vector<IAction>();
	ActionModel	model;
	JTable	table;
	JButton	BTN_ADD;
	JButton	BTN_EDIT;
	JButton	BTN_DEL;
	JButton	BTN_VAL;
	ActionDialog	dlg;
	boolean	bChg = false;
	ResourceBundle	res;

	public ActionPanel( Window owner, SetupParam setup ) {
		this.owner = owner;
		this.setup = setup;
		if ( owner instanceof Frame ) {
			dlg = new ActionDialog( (Frame )owner, setup );
		} else
		if ( owner instanceof Dialog ) {
			dlg = new ActionDialog( (Dialog )owner, setup );
		}
		createControls();
	}

	protected void createControls() {
		res = ResourceBundle.getBundle( "resource.monitor" );
		String text;
		setLayout( new BoxLayout( this, BoxLayout.X_AXIS ));
		model = new ActionModel();
		table = new JTable( model );
		WindowUtility.removeTableEscape( table );
		JScrollPane sp = new JScrollPane( table );
		add( sp );
		table.addMouseListener( new MouseAdapter() {
        	public void mouseClicked( MouseEvent ev ) {
				if ( ev.getClickCount() == 2 ) {
					onEdit();
					ev.consume();
				}
			}
		});
		table.addKeyListener( new KeyAdapter() {
			public void keyPressed( KeyEvent ev ) {
				if ( ev.getKeyChar() == '\n' ) {
					onEdit();
					ev.consume();
				}
			}
		});
		table.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
		TableColumn col;
		col = table.getColumnModel().getColumn( 0 );
		col.setPreferredWidth( 100 );
		col = table.getColumnModel().getColumn( 1 );
		col.setPreferredWidth( 400 );
		col.setCellRenderer( new DescRenderer() );

		JPanel pnl = new JPanel();
		pnl.setLayout( new BoxLayout( pnl, BoxLayout.Y_AXIS ));
		text = res.getString( "action.button1.text" );
		BTN_ADD = new JButton( text );
		BTN_ADD.setMnemonic( KeyEvent.VK_A );
		pnl.add( BTN_ADD );
		text = res.getString( "action.button2.text" );
		BTN_EDIT = new JButton( text );
		BTN_EDIT.setMnemonic( KeyEvent.VK_E );
		pnl.add( BTN_EDIT );
		text = res.getString( "action.button3.text" );
		BTN_DEL = new JButton( text );
		BTN_DEL.setMnemonic( KeyEvent.VK_D );
		pnl.add( BTN_DEL );
		text = res.getString( "action.button4.text" );
		BTN_VAL = new JButton( text );
		BTN_VAL.setMnemonic( KeyEvent.VK_V );
		pnl.add( BTN_VAL );
		add( pnl );
		BTN_ADD.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				onAdd();
			}
		});
		BTN_EDIT.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				onEdit();
			}
		});
		BTN_DEL.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				onDel();
			}
		});
		BTN_VAL.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				onValid();
			}
		});

	}

	public void addActionList( java.util.List<IAction> lstAction ) {
		vecAction.addAll( lstAction );
		model.fireTableDataChanged();
	}
	public void addAction( IAction act ) {
		vecAction.add( act );
		int nRow = vecAction.size() - 1;
		model.fireTableRowsInserted( nRow, nRow );
	}
	public void clearAction() {
		bChg = false;
		vecAction.clear();
		table.clearSelection();
		model.fireTableDataChanged();
	}
	public java.util.List<IAction> listActions() {
		return vecAction;
	}

	protected void onAdd() {
		IAction a = createAction();
		if ( a == null ) {
			return;
		}
		dlg.clearAction();
		dlg.setAction( a );
		dlg.setEnabled( isEnabled());
		WindowUtility.center( this, dlg );
		dlg.setVisible( true );
		if ( dlg.isOk() ) {
			IAction action = dlg.getAction();
			vecAction.add( action );
			int nRow = vecAction.size() - 1;
			model.fireTableRowsInserted( nRow, nRow );
			bChg = true;
		}
	}

	private IAction createAction() {
		Map<String,SetupParam.Registry> mapAction = setup.getActionMap();
		String[] options = new String[ mapAction.size() ];
		int nPos = 0;
		HashMap<String,SetupParam.Registry> mapSel = new HashMap<String,SetupParam.Registry>();
		Iterator<String> itr = mapAction.keySet().iterator();
		while ( itr.hasNext() ) {
			String key = (String )itr.next();
			SetupParam.Registry reg = (SetupParam.Registry )mapAction.get( key );
			mapSel.put( reg.getType(), reg );
			options[nPos++] = reg.getType();
		}
		String title = res.getString( "action.add.title" );
		String msg = res.getString( "action.add.message" );
		String sel = (String )JOptionPane.showInputDialog( owner,
			msg, title, JOptionPane.INFORMATION_MESSAGE,
			null, options, options[0] );
		if ( sel == null ) {
			return null;
		}

		String cls = ((SetupParam.Registry )mapSel.get( sel )).getOrgClass();
		IAction ret = null;
		try {
			ret = (IAction )Class.forName( cls ).newInstance();
			ret.setFilter( new EventFilter() );
		} catch ( Exception ex ) {
			logger.error( "error", ex );
			title = res.getString( "action.err3.title" );
			msg = res.getString( "action.err3.message" );
			JOptionPane.showMessageDialog( this,
				msg + "\n" + ex + "\n" + cls,
				title, JOptionPane.ERROR_MESSAGE );
			return null;
		}
		return ret;
	}

	protected void onEdit() {
		int nPos = table.getSelectedRow();
		if ( table.getRowCount() == 0 || nPos < 0 ) {
			return;
		}
		IAction action = (IAction )vecAction.get( nPos );
		dlg.setAction( action );
		dlg.setEnabled( isEnabled());
		WindowUtility.center( this, dlg );
		dlg.setVisible( true );
		if ( dlg.isOk()) {
			bChg = true;
			model.fireTableRowsUpdated( nPos, nPos );
		}
	}

	protected void onDel() {
		int nPos = table.getSelectedRow();
		if ( table.getRowCount() == 0 || nPos < 0 ) {
			return;
		}
		String title = res.getString( "action.del.title" );
		String msg = res.getString( "action.del.message" );
		int nRet = JOptionPane.showConfirmDialog( this,
			msg, title, JOptionPane.YES_NO_OPTION );
		if ( nRet != JOptionPane.YES_OPTION ) {
			return;
		}
		vecAction.remove( nPos );
		model.fireTableRowsDeleted( nPos, nPos );
		table.clearSelection();
		bChg = true;
	}

	protected void onValid() {
		int nPos = table.getSelectedRow();
		if ( table.getRowCount() == 0 || nPos < 0 ) {
			return;
		}
		IAction action = (IAction )vecAction.get( nPos );
		action.setValid( !action.isValid());
		model.fireTableRowsUpdated( nPos, nPos );
		bChg = true;
	}

	public void setChange( boolean var ) {
		this.bChg = var;
	}
	public boolean isChange() {
		return bChg;
	}

	public void setEnabled( boolean var ) {
		super.setEnabled( var );
		// table.setEnabled( var );
		BTN_ADD.setEnabled( var );
		// BTN_EDIT.setEnabled( var );
		BTN_DEL.setEnabled( var );
		BTN_VAL.setEnabled( var );
	}

	class ActionModel extends AbstractTableModel
	{
		private static final long serialVersionUID = 1L;
		String[] cols = { "Type", "Name" };

		public int getColumnCount() {
			return cols.length;
		}
		public String getColumnName( int col ) {
			return cols[col];
		}

		public int getRowCount() {
			return vecAction.size();
		}
		public Object getValueAt( int row, int col ) {
			IAction action = (IAction )vecAction.get( row );
			switch ( col ) {
			case 0:
				SetupParam.Registry reg = setup.getActionType( action );
				if ( reg == null ) {
					return "(UNKNOWN)";
				}
				return reg.getType();
			case 1: return action.paramString();
			}
			return null;
		}
	}

	public class DescRenderer extends DefaultTableCellRenderer
	{
		private static final long serialVersionUID = 1L;

		public Component getTableCellRendererComponent(
			JTable table, Object obj, boolean isSelected, boolean hasFocus,
			int row, int column)
		{
			JLabel lbl = (JLabel )super.getTableCellRendererComponent(
							table, obj, isSelected, hasFocus, row, column );
			lbl.setToolTipText( obj.toString() );
			return lbl;
		}
	}

}
