package	jp.co.sds_corp.hibiya.monitor;

import jp.co.sds_corp.util.config.IConfigSavable;
import jp.co.sds_corp.util.config.IConfigurable;
import jp.co.sds_corp.util.config.IConfiguration;

/**
* グラフの設定を保持します。
*/
public class GraphSetupParam implements IConfigurable, IConfigSavable
{
	public enum Type {
		/** グラフタイプ：折れ線グラフ */
		LINE,
		/** グラフタイプ：階段グラフ */
		STAIR,
		/** グラフタイプ：積上げグラフ */
		STACK
	};
	public enum Term {
		/** 表示期間：5分 */
		MIN15,
		/** 表示期間：1時間 */
		HOUR,
		/** 表示期間：1日 */
		DATE
	};

	Type	type;	// グラフのタイプ
	Term	term;	// 表示期間

	public void init( IConfiguration config ) throws Exception {
		IConfiguration c = config.getAttributes();
		String var;
		var = c.get( "type" );
		if ( var.equals( "stair" )) {
			type = Type.STAIR;
		} else if ( var.equals( "line" )) {
			type = Type.LINE;
		} else if ( var.equals( "stack" )) {
			type = Type.STACK;
		} else {
			type = Type.LINE;
		}
		var = c.get( "term" );
		if ( var.equals( "5min" )) {
			term = Term.MIN15;
		} else if ( var.equals( "hour" )) {
			term = Term.HOUR;
		} else if ( var.equals( "date" )) {
			term = Term.DATE;
		} else {
			term = Term.MIN15;
		}
	}

	public void saveConfig( IConfiguration config ) {
		switch ( type ) {
		case STAIR: config.setAttribute( "type", "stair" ); break;
		case LINE:  config.setAttribute( "type", "line" ); break;
		case STACK:  config.setAttribute( "type", "stack" ); break;
		}
		switch ( term ) {
		case MIN15: config.setAttribute( "term", "5min" ); break;
		case HOUR:  config.setAttribute( "term", "hour" ); break;
		case DATE:  config.setAttribute( "term", "date" ); break;
		}
	}

	/**
	* グラフのタイプを設定します。
	* @param	var		設定するデータ
	*/
	public void setType( Type var ) {
		this.type = var;
	}
	/**
	* グラフのタイプを取得します。
	* @return	グラフのタイプ
	*/
	public Type getType() {
		return type;
	}

	/**
	* 表示期間を設定します。
	* @param	var		設定するデータ
	*/
	public void setTerm( Term var ) {
		this.term = var;
	}
	/**
	* 表示期間を取得します。
	* @return	表示期間
	*/
	public Term getTerm() {
		return term;
	}

}
